@@ -1,5 +1,6 @@
 from .utils import (
     TransferProgress,
+    slugify_string,
 )
 from .server import (
     ServerAPIBase,
@@ -7,7 +8,14 @@ from .server import (
 
 from .server_api import (
     ServerAPI,
+    ServiceContext,
 
+    init_service,
+
+    is_connection_created,
+    close_connection,
+    change_token,
+    set_environments,
     get_server_api_connection,
 
     get_base_url,
@@ -25,6 +33,11 @@ from .server_api import (
     patch,
     delete,
 
+    get_event,
+    dispatch_event,
+    update_event,
+    enroll_event_job,
+
     download_file,
     upload_file,
 
@@ -99,11 +112,18 @@ from .server_api import (
 
 __all__ = (
     "TransferProgress",
+    "slugify_string",
 
     "ServerAPIBase",
 
     "ServerAPI",
+    "ServiceContext",
+    "init_service",
 
+    "is_connection_created",
+    "close_connection",
+    "change_token",
+    "set_environments",
     "get_server_api_connection",
 
     "get_base_url",
@@ -121,6 +141,11 @@ __all__ = (
     "patch",
     "delete",
 
+    "get_event",
+    "dispatch_event",
+    "update_event",
+    "enroll_event_job",
+
     "download_file",
     "upload_file",
 
