@@ -75,13 +75,32 @@ class RequestTypes:
 class RestApiResponse(object):
     """API Response."""
 
-    def __init__(self, status_code, data=None, content=None, headers=None):
-        if data is None:
-            data = {}
+    def __init__(self, response, data=None):
+        if response is None:
+            status_code = 500
+        else:
+            status_code = response.status_code
+        self._response = response
         self.status = status_code
-        self.data = data
-        self.content = content
-        self.headers = headers or {}
+        self._data = data
+
+    @property
+    def orig_response(self):
+        return self._response
+
+    @property
+    def headers(self):
+        return self._response.headers
+
+    @property
+    def data(self):
+        if self._data is None:
+            self._data = self._response.json()
+        return self._data
+
+    @property
+    def content(self):
+        return self._response.content
 
     @property
     def content_type(self):
@@ -95,6 +114,12 @@ class RestApiResponse(object):
     def status_code(self):
         return self.status
 
+    def raise_for_status(self):
+        self._response.raise_for_status()
+
+    def __enter__(self, *args, **kwargs):
+        return self._response.__enter__(*args, **kwargs)
+
     def __contains__(self, key):
         return key in self.data
 
@@ -448,26 +473,22 @@ class ServerAPIBase(object):
 
         except ConnectionRefusedError:
             new_response = RestApiResponse(
-                500,
+                None,
                 {"detail": "Unable to connect the server. Connection refused"}
             )
         except requests.exceptions.ConnectionError:
             new_response = RestApiResponse(
-                500,
+                None,
                 {"detail": "Unable to connect the server. Connection error"}
             )
         else:
             content_type = response.headers.get("Content-Type")
             if content_type == "application/json":
                 try:
-                    new_response = RestApiResponse(
-                        response.status_code,
-                        response.json(),
-                        headers=response.headers,
-                    )
+                    new_response = RestApiResponse(response)
                 except JSONDecodeError:
                     new_response = RestApiResponse(
-                        500,
+                        None,
                         {
                             "detail": "The response is not a JSON: {}".format(
                                 response.text)
@@ -475,17 +496,10 @@ class ServerAPIBase(object):
                     )
 
             elif content_type in ("image/jpeg", "image/png"):
-                new_response = RestApiResponse(
-                    response.status_code,
-                    headers=response.headers,
-                    content=response.content
-                )
+                new_response = RestApiResponse(response)
 
             else:
-                new_response = RestApiResponse(
-                    response.status_code,
-                    headers=response.headers,
-                )
+                new_response = RestApiResponse(response)
 
         self.log.debug("Response {}".format(str(new_response)))
         return new_response
@@ -555,6 +569,119 @@ class ServerAPIBase(object):
     def delete(self, entrypoint, **kwargs):
         return self.raw_delete(entrypoint, params=kwargs)
 
+    def get_event(self, event_id):
+        """Receive full event data by id.
+
+        Events received using event server do not contain full information. To
+        get the full event information is required to receive it explicitly.
+
+        Args:
+            event_id (str): Id of event.
+
+        Returns:
+            Dict[str, Any]: Full event data.
+        """
+
+        response = self.get("events/{}".format(event_id))
+        response.raise_for_status()
+        return response.data
+
+    def update_event(
+        self,
+        event_id,
+        sender=None,
+        project_name=None,
+        status=None,
+        description=None,
+        summary=None,
+        payload=None
+    ):
+        kwargs = {}
+        for key, value in (
+            ("sender", sender),
+            ("projectName", project_name),
+            ("status", status),
+            ("description", description),
+            ("summary", summary),
+            ("payload", payload),
+        ):
+            if value is not None:
+                kwargs[key] = value
+        response = self.patch(
+            "events/{}".format(event_id),
+            **kwargs
+        )
+        response.raise_for_status()
+
+    def dispatch_event(
+        self,
+        topic,
+        sender=None,
+        event_hash=None,
+        project_name=None,
+        username=None,
+        dependencies=None,
+        description=None,
+        summary=None,
+        payload=None,
+        finished=True,
+        store=True,
+    ):
+        if summary is None:
+            summary = {}
+        if payload is None:
+            payload = {}
+        event_data = {
+            "topic": topic,
+            "sender": sender,
+            "hash": event_hash,
+            "project": project_name,
+            "user": username,
+            "dependencies": dependencies,
+            "description": description,
+            "summary": summary,
+            "payload": payload,
+            "finished": finished,
+            "store": store,
+        }
+        if self.post("events", **event_data):
+            self.log.debug("Dispatched event {}".format(topic))
+            return True
+        self.log.error("Unable to dispatch event {}".format(topic))
+        return False
+
+    def enroll_event_job(
+        self,
+        source_topic,
+        target_topic,
+        sender,
+        description=None,
+        sequential=None
+    ):
+        """Enroll job based on events.
+
+        Args:
+            source_topic (str): Topic
+        """
+
+        kwargs = {
+            "sourceTopic": source_topic,
+            "targetTopic": target_topic,
+            "sender": sender,
+        }
+        if sequential is not None:
+            kwargs["sequential"] = sequential
+        if description is not None:
+            kwargs["description"] = description
+        response = self.post("enroll", **kwargs)
+        if response.status_code == 204:
+            return None
+        elif response.status_code >= 400:
+            self.log.error(response.text)
+            return None
+
+        return response.data
+
     def _download_file(self, url, filepath, chunk_size, progress):
         dst_directory = os.path.dirname(filepath)
         if not os.path.exists(dst_directory):
@@ -573,7 +700,7 @@ class ServerAPIBase(object):
                 progress.set_content_size(response.headers["Content-length"])
                 for chunk in response.iter_content(chunk_size=chunk_size):
                     f_stream.write(chunk)
-                    progress.add_downloaded_chunk(len(chunk))
+                    progress.add_transferred_chunk(len(chunk))
 
     def download_file(self, endpoint, filepath, chunk_size=None, progress=None):
         """Download file from AYON server.
@@ -618,7 +745,7 @@ class ServerAPIBase(object):
             raise
 
         finally:
-            progress.set_done()
+            progress.set_transfer_done()
 
     def _upload_file(self, url, filepath, progress):
         kwargs = {}
@@ -635,7 +762,7 @@ class ServerAPIBase(object):
             progress.set_content_size(size)
             response = post_func(url, data=stream, **kwargs)
         response.raise_for_status()
-        progress.set_downloaded_size(size)
+        progress.set_transferred_size(size)
 
     def upload_file(self, endpoint, filepath, progress=None):
         """Upload file to server.
@@ -673,7 +800,7 @@ class ServerAPIBase(object):
             raise
 
         finally:
-            progress.set_done()
+            progress.set_transfer_done()
 
     def trigger_server_restart(self):
         result = self.post("system/restart")
@@ -889,8 +1016,7 @@ class ServerAPIBase(object):
         if details:
             endpoint += "?details=1"
         response = self.get(endpoint)
-        if response.status != 200:
-            raise ServerError(response.text)
+        response.raise_for_status()
         return response.data
 
     def download_addon_private_file(
@@ -1106,8 +1232,7 @@ class ServerAPIBase(object):
         result = self.get(
             "addons/{}/{}/settings".format(addon_name, addon_version)
         )
-        if result.status != 200:
-            raise ServerError(result.text)
+        result.raise_for_status()
         return result.data
 
     def get_addon_project_settings(
@@ -1118,8 +1243,7 @@ class ServerAPIBase(object):
                 addon_name, addon_version, project_name
             )
         )
-        if result.status != 200:
-            raise ServerError(result.text)
+        result.raise_for_status()
         return result.data
 
     def get_addon_settings(self, addon_name, addon_version, project_name=None):
@@ -1160,7 +1284,7 @@ class ServerAPIBase(object):
                 returned if 'None' is passed.
 
         Returns:
-            List[Dict[str, Any]]: List of available projects.
+            Generator[Dict[str, Any]]: Available projects.
         """
 
         for project_name in self.get_project_names(active, library):
@@ -1223,8 +1347,7 @@ class ServerAPIBase(object):
             ]))
 
         response = self.get("projects{}".format(query), **query_keys)
-        # TODO check status
-        response.status
+        response.raise_for_status()
         data = response.data
         project_names = []
         if data:
@@ -1248,7 +1371,7 @@ class ServerAPIBase(object):
                 on entity will have 'None' value.
 
         Returns:
-            List[Dict[str, Any]]: List of queried projects.
+            Generator[Dict[str, Any]]: Queried projects.
         """
 
         if fields is None:
@@ -1357,11 +1480,11 @@ class ServerAPIBase(object):
                 if 'None' is passed.
 
         Returns:
-            Iterable[dict[str, Any]]: Queried folder entities.
+            Generator[dict[str, Any]]: Queried folder entities.
         """
 
         if not project_name:
-            return []
+            return
 
         filters = {
             "projectName": project_name
@@ -1369,25 +1492,25 @@ class ServerAPIBase(object):
         if folder_ids is not None:
             folder_ids = set(folder_ids)
             if not folder_ids:
-                return []
+                return
             filters["folderIds"] = list(folder_ids)
 
         if folder_paths is not None:
             folder_paths = set(folder_paths)
             if not folder_paths:
-                return []
+                return
             filters["folderPaths"] = list(folder_paths)
 
         if folder_names is not None:
             folder_names = set(folder_names)
             if not folder_names:
-                return []
+                return
             filters["folderNames"] = list(folder_names)
 
         if parent_ids is not None:
             parent_ids = set(parent_ids)
             if not parent_ids:
-                return []
+                return
             if None in parent_ids:
                 # Replace 'None' with '"root"' which is used during GraphQl
                 #   query for parent ids filter for folders without folder
@@ -1448,7 +1571,7 @@ class ServerAPIBase(object):
         own_attributes=False
     ):
         if not project_name:
-            return []
+            return
 
         filters = {
             "projectName": project_name
@@ -1457,25 +1580,25 @@ class ServerAPIBase(object):
         if task_ids is not None:
             task_ids = set(task_ids)
             if not task_ids:
-                return []
+                return
             filters["taskIds"] = list(task_ids)
 
         if task_names is not None:
             task_names = set(task_names)
             if not task_names:
-                return []
+                return
             filters["taskNames"] = list(task_names)
 
         if task_types is not None:
             task_types = set(task_types)
             if not task_types:
-                return []
+                return
             filters["taskTypes"] = list(task_types)
 
         if folder_ids is not None:
             folder_ids = set(folder_ids)
             if not folder_ids:
-                return []
+                return
             filters["folderIds"] = list(folder_ids)
 
         if not fields:
@@ -1633,6 +1756,20 @@ class ServerAPIBase(object):
             for folder in folders
         }
 
+    def _filter_subset(
+        self, project_name, subset, active, own_attributes, use_rest
+    ):
+        if active is not None and subset["active"] is not active:
+            return None
+
+        if use_rest:
+            subset = self.get_rest_subset(project_name, subset["id"])
+
+        if own_attributes:
+            fill_own_attribs(subset)
+
+        return subset
+
     def get_subsets(
         self,
         project_name,
@@ -1645,24 +1782,24 @@ class ServerAPIBase(object):
         own_attributes=False
     ):
         if not project_name:
-            return []
+            return
 
         if subset_ids is not None:
             subset_ids = set(subset_ids)
             if not subset_ids:
-                return []
+                return
 
         filter_subset_names = None
         if subset_names is not None:
             filter_subset_names = set(subset_names)
             if not filter_subset_names:
-                return []
+                return
 
         filter_folder_ids = None
         if folder_ids is not None:
             filter_folder_ids = set(folder_ids)
             if not filter_folder_ids:
-                return []
+                return
 
         # This will disable 'folder_ids' and 'subset_names' filters
         #   - maybe could be enhanced in future?
@@ -1676,7 +1813,7 @@ class ServerAPIBase(object):
                     filter_subset_names |= set(names)
 
             if not filter_subset_names or not filter_folder_ids:
-                return []
+                return
 
         # Convert fields and add minimum required fields
         if fields:
@@ -1720,35 +1857,30 @@ class ServerAPIBase(object):
         parsed_data = query.query(self)
 
         subsets = parsed_data.get("project", {}).get("subsets", [])
-        if active is not None or own_attributes:
-            _subsets = []
-            for subset in subsets:
-                if active is not None and subset["active"] is not active:
-                    continue
-
-                if use_rest:
-                    subset = self.get_rest_subset(project_name, subset["id"])
-
-                if own_attributes:
-                    fill_own_attribs(subset)
-                _subsets.append(subset)
-            subsets = _subsets
-
         # Filter subsets by 'names_by_folder_ids'
         if names_by_folder_ids:
             subsets_by_folder_id = collections.defaultdict(list)
             for subset in subsets:
-                folder_id = subset["folderId"]
-                subsets_by_folder_id[folder_id].append(subset)
+                filtered_subset = self._filter_subset(
+                    project_name, subset, active, own_attributes, use_rest
+                )
+                if filtered_subset is not None:
+                    folder_id = filtered_subset["folderId"]
+                    subsets_by_folder_id[folder_id].append(filtered_subset)
 
-            filtered_subsets = []
             for folder_id, names in names_by_folder_ids.items():
                 for folder_subset in subsets_by_folder_id[folder_id]:
                     if folder_subset["name"] in names:
-                        filtered_subsets.append(subset)
-            subsets = filtered_subsets
+                        yield folder_subset
+
+        else:
+            for subset in subsets:
+                filtered_subset = self._filter_subset(
+                    project_name, subset, active, own_attributes, use_rest
+                )
+                if filtered_subset is not None:
+                    yield filtered_subset
 
-        return list(subsets)
 
     def get_subset_by_id(
         self,
@@ -1845,7 +1977,7 @@ class ServerAPIBase(object):
                 if 'None' is passed.
 
         Returns:
-            List[Dict[str, Any]]: Queried version entities.
+            Generator[Dict[str, Any]]: Queried version entities.
         """
 
         if not fields:
@@ -1872,24 +2004,24 @@ class ServerAPIBase(object):
         if version_ids is not None:
             version_ids = set(version_ids)
             if not version_ids:
-                return []
+                return
             filters["versionIds"] = list(version_ids)
 
         if subset_ids is not None:
             subset_ids = set(subset_ids)
             if not subset_ids:
-                return []
+                return
             filters["subsetIds"] = list(subset_ids)
 
         # TODO versions can't be used as fitler at this moment!
         if versions is not None:
             versions = set(versions)
             if not versions:
-                return []
+                return
             filters["versions"] = list(versions)
 
         if not hero and not standard:
-            return []
+            return
 
         queries = []
         # Add filters based on 'hero' and 'standard'
@@ -2158,7 +2290,7 @@ class ServerAPIBase(object):
                 passed.
 
         Returns:
-            List[Dict[str, Any]]: Queried representation entities.
+            Generator[Dict[str, Any]]: Queried representation entities.
         """
 
         if not fields:
@@ -2183,7 +2315,7 @@ class ServerAPIBase(object):
         if representation_ids is not None:
             representation_ids = set(representation_ids)
             if not representation_ids:
-                return []
+                return
             filters["representationIds"] = list(representation_ids)
 
         version_ids_filter = None
@@ -2196,18 +2328,18 @@ class ServerAPIBase(object):
                 representaion_names_filter |= set(names)
 
             if not version_ids_filter or not representaion_names_filter:
-                return []
+                return
 
         else:
             if representation_names is not None:
                 representaion_names_filter = set(representation_names)
                 if not representaion_names_filter:
-                    return []
+                    return
 
             if version_ids is not None:
                 version_ids_filter = set(version_ids)
                 if not version_ids_filter:
-                    return []
+                    return
 
         if version_ids_filter:
             filters["versionIds"] = list(version_ids_filter)
@@ -2332,19 +2464,19 @@ class ServerAPIBase(object):
         if task_ids is not None:
             task_ids = set(task_ids)
             if not task_ids:
-                return []
+                return
             filters["taskIds"] = list(task_ids)
 
         if paths is not None:
             paths = set(paths)
             if not paths:
-                return []
+                return
             filters["paths"] = list(paths)
 
         if workfile_ids is not None:
             workfile_ids = set(workfile_ids)
             if not workfile_ids:
-                return []
+                return
             filters["workfileIds"] = list(workfile_ids)
 
         if not fields:
@@ -2722,4 +2854,5 @@ class ServerAPIBase(object):
                     operation_id,
                     json.dumps(body_by_id[operation_id], indent=4),
                     op_result["error"],
-                ))
\ No newline at end of file
+                ))
+
